 _CLKMODE = XTAL1 + PLL16X ' The clock frequency is equal to 96MHz = 6MHz * 16X PLL.
  _XINFREQ = 5_000_000 '

  _STACK = 640 ' Reserve some space for the stack and free.
  _FREE = 2 '
  _CAM_XCLK_PIN = 11 ' Camera signaling pins.
  _CAM_PCLK_PIN = 10 '
  _CAM_VSYNC_PIN = 12 '
  _CAM_HREF_PIN = 13 '
  _I2C_CLOCK_PIN = 14 ' I2C bus pins. Must use 4.7K pullups to 3.3v
  _I2C_DATA_PIN = 15 '

  _CAM_PWDN_PIN = 8 ' Camera state control pins.
  _CAM_RESET_PIN = -1 '

  _SCCB_CAMERA_WRITE_ADDRESS  = $60    ' SCCB/I2C address: corresponds to write address $ and read address $61
  _SCCB_CAMERA_READ_ADDRESS = $61
  CamXVCLKFreq  =2_000_000     ' XVCLK frequency determines SCCB (Serial Camera Control Bus) speed, OVM7690 data sheet section 1.2.9
  SerRxPin      =31            ' Serial interface via Prop Clip, IN from user
  SerTxPin      = 30              '                                 OUT to user

Var
   long mydata,time,Counter1,Counter2,reg,testval,framerate,pixelclocks,pixelsperline,linesperframe,i
   long VsyncH,VsyncL
   long start
   byte databuffer[20000]
OBJ

    freq          : "Synth"                               ' Frequency synthesizer (included w/ Parallax Propeller Tool)
    ser           :"FullDuplexSerial"
PUB I2C_test |cmd
'' Start cog if not started, and initialize the devices on the I2C bus
  'ser.dbprintf1(string("i2c initialize %d\n"), SCL)
 ' outa[14]:=dira[14]:=1
          ' Launch the cog with a pointer to the parameters
  i:=0
 ' outa[ _CAM_PWDN_PIN]:=dira[ _CAM_PWDN_PIN]:=1
  waitcnt(clkfreq/10 + cnt)
  outa[ _CAM_PWDN_PIN]:=dira[ _CAM_PWDN_PIN]:=0
  dira[14]:=0
  dira[9]:=outa[9]:=1
  freq.Synth("A",  _CAM_XCLK_PIN, CamXVCLKFreq)   ' Configure Counter A to synthesize input clock (XVCLK) for OVM7690
  ser.start(SerRxPin,SerTxPin,0,230400)
  ser.rxflush
  start:=0
  cognew(@entry, @start)
 ' waitcnt(clkfreq + cnt)
  SCCBWriteRegister($12,%10000000)
  waitcnt(clkfreq/10 + cnt)
  Set_Registers
  ser.str(string("OV2640 reset",13))
  repeat

    repeat until (cmd >1)
      cmd := ser.Rx               ' Wait here to receive a byte
   ' ser.str(string("command="))
    'ser.tx(cmd)
    'ser.tx(13)
    case cmd
      "L", "l": GetRegisters                'Get Register Settings
      "D","d": DefineSignal
      "T","t": transmit
      "C", "c": capture
    cmd:=0
PRI GetSigAv(seconds)
    time:=cnt
    phsb := 0
    waitcnt(Time += clkfreq*seconds)                            ' 1 Second Wait
    Counter1 := phsb                                   ' Get Counter A Value
                               ' Get Counter B Value
                                              ' Reset Counter A
                                          ' Reset Counter B

    Counter1:=Counter1/seconds
    return counter1
Pri Capture
    start:=1
PRI GetRegisters
  reg := $00
  ser.str(string("Register     Val   Binary",13))
  repeat until (reg>$FF)


    SCCBReadRegister(reg,@mydata)
    ser.str(string(" $"))
    ser.hex(reg,2)
    ser.str(string(" = "))
    ser.dec(reg)
    ser.str(string("       "))
    ser.hex(mydata,2)
    ser.str(string("   "))
    ser.bin(mydata,8)
    ser.str(string("   ",13,10))

    reg++
  ser.rxflush
PRI SCCBWriteRegister(register, data) ' Writes a camera register.

' Notes:
'
' Register - The register number to write.
' Data - The value to write.
'
' Returns true on success and false on failure.

  I2CStart
  result := I2CWrite(_SCCB_CAMERA_WRITE_ADDRESS)
  result and= I2CWrite(register)
  result and= I2CWrite(data)
  I2CStop

PRI SCCBReadRegister(register, longDataAddress) ' Reads a camera register.

' Notes:
'
' Register - The register number to read.
' LongDataAddress - The address of the value to store data to.
'
' Returns true on success and false on failure.

  I2CStart
  result := I2CWrite(_SCCB_CAMERA_WRITE_ADDRESS)
  result and= I2CWrite(register)
  I2CStop
  if(result)

    I2CStart
    result := I2CWrite(_SCCB_CAMERA_READ_ADDRESS)
    long[longDataAddress] := I2CRead(0)
    I2CStop

PRI I2CWrite(data) ' Write out data.

' Notes:
'
' Data - The 8 bit packet to transmit.
'
' Returns true if the receiving device ACK'ed and false if not.

  data := ((!data) >< 8)

  repeat 8 ' Write out all 8 data bits. Leave with clock low.

    dira[_I2C_DATA_PIN] := data
    dira[_I2C_CLOCK_PIN] := 0
    dira[_I2C_CLOCK_PIN] := 1
    data >>= 1

  ' Leave with clock low and data low.

  dira[_I2C_DATA_PIN] := 0
  dira[_I2C_CLOCK_PIN] := 0
  result := not(ina[_I2C_DATA_PIN])
  dira[_I2C_CLOCK_PIN] := 1
  dira[_I2C_DATA_PIN] := 1

PRI I2CRead(aknowledge) ' Read in data.

' Notes:
'
' Aknowledge - True to send the transmitting device an ACK and false to not send a NCK.
'
' Returns the received 8 bit packet.

  dira[_I2C_DATA_PIN] := 0

  repeat 8 ' Read in all 8 data bits. Leave with clock low.

    result <<= 1
    dira[_I2C_CLOCK_PIN] := 0
    result |= ina[_I2C_DATA_PIN]
    dira[_I2C_CLOCK_PIN] := 1

  ' Leave with the clock low and the data low.

  dira[_I2C_DATA_PIN] := (not(not(aknowledge)))
  dira[_I2C_CLOCK_PIN] := 0
  dira[_I2C_CLOCK_PIN] := 1
  dira[_I2C_DATA_PIN] := 1

PRI I2CStart ' Cause the I2C start condition.

  outa[_I2C_DATA_PIN] := 0
  dira[_I2C_DATA_PIN] := 1

  outa[_I2C_CLOCK_PIN] := 0
  dira[_I2C_CLOCK_PIN] := 1

PRI I2CStop ' Cause the I2C stop condition.

  dira[_I2C_CLOCK_PIN] := 0
  dira[_I2C_DATA_PIN] := 0
Pri Set_Registers
  ' waitcnt(clkfreq*10 + cnt)
'  byte	$ff, $0
'byte    $2c, $ff
'byte	$2e, $df
'byte	$ff, $1
  SCCBWriteRegister($FF,$0)

  SCCBWriteRegister($2C,$FF)
  SCCBWriteRegister($2E,$DF)
 ' SCCBWriteRegister($2E,$00)




  SCCBWriteRegister($C2,%00001100)'yuv
  SCCBWriteRegister($12,%10000000)'yuv en
  SCCBWriteRegister($FF,$1)
  SCCBWriteRegister($32,%11000000) 'pixel clock divider
Pri DefineSignal


  ser.str(string(13,"Camera Central OV2640!",10))
  ser.str(string("Please wait... gathering signal characteristics",13,10))
  ser.tx(10)
  ser.tx(13)
  ser.str(string(" External Clock= "))
  ser.dec(CamXVCLKFreq)
  ser.str(string(13," ",10,13))

  ctrb := %01010<< 26  + 10 '12          ' Counter B: POSEDGE on p10
  frqb := 1
  ser.str(string(" Pixel Clock Hz= "))
  pixelclocks:=GetSigAv(3)
  ser.dec(pixelclocks)
  ser.tx(10)
  ser.tx(13)
  ctrb := %01010<< 26  + _CAM_VSYNC_PIN '12          ' Counter B: POSEDGE on vsync
  ser.str(string(" Frame Rate Hz= "))
  framerate:=GetSigAv(3)
  ser.dec(framerate)
  ser.tx(10)
  ser.tx(13)
  ctrb := %01010<< 26  + _CAM_HREF_PIN          ' Counter B: POSEDGEs
  ser.str(string(" Lines Per Frame = "))
  testval:=GetSigAv(3)
  linesperframe:=testval/framerate
  ser.dec(linesperframe)
  ser.tx(10)
  ser.tx(13)
  ser.str(string(" Clocks Per Line ="))
  PixelsperLine:= pixelclocks/(framerate*linesperframe)
  ser.dec(pixelsperline)
  ser.tx(10)                                          ' Clear Serial Terminal Screen
  ser.tx(13)
  Time := cnt                                           ' Current cnt Value


Pub transmit | a,w
  w:=0
  a:=127
  repeat nlines 'lines
    'waitcnt(clkfreq/10 + cnt)
    repeat ppl 'pixels
      ser.dec(databuffer[w])

     ' ser.dec(a)
      ser.tx(32)
      w++
    ser.tx(13)



DAT
                        org     0
entry
mainloop                rdlong   _cmd,par  'wait for command to start... anything other than zero
                        cmp   _cmd,#0  wz
              if_e      jmp #mainloop

                        mov   ptraddr,par
                        wrlong zero,ptraddr
                        add   ptraddr,#4
                        mov   linecnt,nlines
                        mov   pixelcnt,ppl

                        call  #wait_vsync_h   'waits for vsync high
                        call  #wait_vsync_l   'waits for vsync to go low
                        mov   temp2,windowy

skiplines               call  #wait_href_h'wait until href is high  'skip down to start of frame
                        call #wait_href_l
                        djnz  temp2,#skiplines


getframe                mov   pixelcnt,ppl

                        call  #wait_href_h'wait until href is high  'skip 4 lines...
                        call #wait_href_l
                        call  #wait_href_h'wait until href is high  '
                        call #wait_href_l
                        call  #wait_href_h'wait until href is high  '

                        call #wait_href_l
                        call  #wait_href_h'wait until href is high  '
                                         call #wait_href_l
                        call  #wait_href_h'wait until href is high  '

                        call #wait_href_l
                        call  #wait_href_h'wait until href is high  '
                        call  #wait_href_h'wait until href is high  '
                        call #wait_href_l
                        call  #wait_href_h'wait until href is high  '

                        call #wait_href_l
                        call  #wait_href_h'wait until href is high  '
                                 call #wait_href_l
                        call  #wait_href_h'wait until href is high  '

                        call #wait_href_l
                        call  #wait_href_h'wait until href is high  '

getline                 call  #wait_PCLK_l 'wait until pixelclock is low

                        mov   temp,ina

                        wrbyte temp,ptraddr
                        call  #wait_PCLK_h
                        add ptraddr, #1

                        call #wait_PCLK_l 'wait until pixelclock is low
                     '   mov   temp,ina

                     '   wrbyte temp,ptraddr
                        call  #wait_PCLK_h
                     '   add ptraddr, #1
                        djnz pixelcnt,#getline
                        call #wait_href_l
                        djnz linecnt,#getframe



                        jmp #mainloop


wait_vsync_h            mov     regA, ina               ' Get value of port A                   wait vsync high        4 [4]
                        and     regA, CamVSYNC    wz   ' Look at VSYNC pin only                        4 [8]
              if_z      jmp     #wait_vsync_h

wait_vsync_h_ret        ret

wait_vsync_l            mov     regA, ina               ' Get value of port A                         wait vsynch low
                        and     regA, CamVSYNC    wz
              if_nz     jmp     #wait_vsync_l

wait_vsync_l_ret        ret

wait_href_h             mov     regA, ina               ' Get value of port A                   wait vsync high        4 [4]
                        and     regA, CamHRef    wz   ' Look at Href pin only                        4 [8]
              if_z      jmp     #wait_href_h

wait_href_h_ret        ret

wait_href_l             mov     regA, ina               ' Get value of port A                   wait vsync high        4 [4]
                        and     regA, CamHRef    wz   ' Look at Href pin only                        4 [8]
              if_nz      jmp     #wait_href_l

wait_href_l_ret        ret

wait_PCLK_h            mov regA,ina
                       and regA, PCLK wz
              if_z     jmp #wait_PCLK_h
wait_PCLK_h_ret        ret

wait_PCLK_l              mov regA,ina
                        and regA, PCLK wz
              if_nz     jmp #wait_PCLK_l
wait_PCLK_l_ret         ret
                           '






'constants
CamVSYNC               long    %00000000_00000000_00010000_00000000     ' Mask: P12
CamHRef                long    %00000000_00000000_00100000_00000000      'mask: p13
PCLK                   long    %00000000_00000000_00000100_00000000    'mask: p10
mycon                  long 565
ppl                    long 200
nlines                 long 100
testlong               long  %00001000_00000000_00000100_11111111
zero                   long 0
windowy                long 100
'var

regA          res 1
temp1         res 1
temp2         res 1
timer1        res 1
temp          res 1
VsyncHaddr    res 1
VsyncLaddr    res 1
ptraddr       res 1
linecnt       res 1
pixelcnt      res 1
_cmd           res 1

